/* patricia.h
 * Copyright 2007 (c) Adrian Sai-wah Tam
 * Tue Apr 17 14:13:24 HKT 2007
 *
 * Header file to patricia.c
 */
#ifndef __PATRICIA_H__
#define __PATRICIA_H__

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Structure of a Patricia tree node */
typedef struct patricia_node {
	char*			str;
	int			len;
	void*			data;
	struct patricia_node*	left;
	struct patricia_node*	right;
} patricia_node;

/* Compare bit vectors s1 and s2 for at most n bits, just return Yes (1) or No (0) */
int patricia_bitncmp(const char*s1,const char*s2,int n);

/* Extract the n-th bit from the bit vector s. MSB is defined as the 1st bit. */
int patricia_extractbit(const char*s,int n);

/* Compare bit vectors s1 and s2 for the highest number of equal bits */
int patricia_getmatchlen(const char*s1,const char*s2);

/* Measure the length of a string in number of bits */
int patricia_strlen(const char* const s);

/* Compare two strings to see if they are equal */
int patricia_strcmp(const char* s, const char* t);

/* Create a new Patricia tree node with initialization */
patricia_node* patricia_new(char* str, void* data);

/* Copy the content of a Patricia tree node to another */
void patricia_copy(patricia_node* a,const patricia_node* b);

/* Search for a string in the Patricia tree and return the maximal match */
patricia_node* patricia_maxmatch(patricia_node* tree, char* str);

/* Search for a string in the Patricia tree and return the closest node */
patricia_node* patricia_search(patricia_node* tree, char* str);

/* Given a Patricia tree, put the data block into the tree with the specified string as the key */
void patricia_add(patricia_node** tree, char* str, void* data);

/* Given a Patricia tree, remove the node as indicated by the key */
void patricia_del(patricia_node** tree, char* str);

#ifdef UNITTEST
#include <stdio.h>
#include <string.h>
#include <assert.h>

/* Check if a tree is well-formed */
int patricia_validate(patricia_node* tree);

/* Walk through the Patricia tree */
void patricia_printcompact(patricia_node* tree);

/* Walk through the Patricia tree, and print in a tree form */
void patricia_print(patricia_node* tree,int n);
#endif

#ifdef __cplusplus
}
#endif

#endif
