#ifdef __cplusplus
extern "C" {
#endif

#include "patricia_helper.h"

/* Compare bit vectors s1 and s2 for at most n bits, just return Yes (1) or No (0) */
int patricia_bitncmp(const char*s1,const char*s2,int n)
{
	char mask = (char)(-1);				/* Assumed signed, two's complement representation */
	while((unsigned)n>=8*sizeof(char)) {		/* If we have to check for more than a char */
		if (*s1==*s2) {				/* use character equality to check */
			if (*s1 == '\0') return 0;	/* return equal if we exhausted the string */
			if (n-=8*sizeof(char)) {	/* shift for 8*sizeof(char) bits */
				s1++; s2++;
				continue;		/* continue if we still have bits to match */
			};
			return 0;			/* return equal if we exhaused n */
		};
		return (int)*s1 ^ (int)*s2;		/* return XOR of two char if they don't match */
	};
	return (((int)*s1 ^ (int)*s2) & (int)(mask<<(8*sizeof(char)-n)));
							/* partial char: compare the first n MSBs */
}

/* Extract the n-th bit from the bit vector s. MSB is defined as the 1st bit. */
int patricia_extractbit(const char*s,int n)
{
	n--;
	s += n/(8*sizeof(char));		/* Shift to the correct byte */
	n = n%(8*sizeof(char))+1;		/* Shift to the correct byte */
	return (unsigned)(*s) & (0x01<<(8*sizeof(char)-n));
}

/* Compare bit vectors s1 and s2 for the highest number of equal bits */
int patricia_getmatchlen(const char*s1,const char*s2)
{
	char c;
	int n;
	for (n=0; *s1==*s2; n+=8*sizeof(char)) {	/* Count for full chars */
		s1++; s2++;				/* Shift back the last unmatched byte */
	};
	c = *s1 ^ *s2;					/* Count for partial char, assumed 1 char = 1 byte */
	if (c & '\x80') n += 0;				/* n += highest asserted bit of the XOR */
	else if (c & '\x40') n+= 1;
	else if (c & '\x20') n+= 2;
	else if (c & '\x10') n+= 3;
	else if (c & '\x08') n+= 4;
	else if (c & '\x04') n+= 5;
	else if (c & '\x02') n+= 6;
	else if (c & '\x01') n+= 7;
	return n;
}

/* Measure the length of a string in number of bits */
int patricia_strlen(const char* const s)
{
	int i;
	for(i=0;s[i];i++);
	return i*8*sizeof(char);
};

/* Compare two strings to see if they are equal */
int patricia_strcmp(const char* s, const char* t)
{
	for(;*s && *s==*t; s++, t++);
	return *s ^ *t;
};

#ifdef __cplusplus
}
#endif
